<?php

/**
 * spamcleaner.php
 *
 * Helps an admin to clean up spam in Moodle
 *
 * @version $Id: spamcleaner.php,v 1.12 2008/11/11 05:09:44 dongsheng Exp $
 * @authors Dongsheng Cai, Martin Dougiamas, Amr Hourani
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 */


// Description:
// -----------
//
// This script is designed to help Moodle administrators delete accounts 
// that have been abused by spammers.  Generally these spammers fill their 
// profiles with links, porn and other undesirable content.  This script
// allows you to search user profiles and other areas for particular words 
// and other things, then delete those profiles when found.
// 
// Future versions of Moodle will include this as a standard tool under the 
// admin menu.
// 
// 
// 
// Installation:
// ------------
// 
// 1. Place spamcleaner.php in the top directory of your Moodle site.
// 
// 2. Visit http://yoursite/spamcleaner.php
// 
//
// Further info:
// ------------
// Please read this to find out how to prevent spammers from doing this in the first place:
// 
//     http://docs.moodle.org/en/Reducing_spam_in_Moodle
//
// If you have improvements / ideas / patches / bug fixes please post them here:
//
//     http://tracker.moodle.org/browse/MDL-17144

// Configuration

// List of known spammy keywords, please add more here

$autokeywords = array(
                    "furniture",
                    "Furniture",
                    "Credit",
                    "credit",
                    "Fuck",
                    "Casino",
                    "Poker",
                    "Porn",
                    "Viagra",
                    "VIAGRA",
                    "<img",
                    "fuck",
                    "casino",
                    "porn",
                    "xxx",
                    "cialis",
                    "viagra",
                    "poker",
                    "warcraft"
                );


/////////////////////////////////////////////////////////////////////////////////

require_once('config.php');

require_js(array('yui_dom-event', 'yui_connection', 'yui_json'));
require_login();
require_capability('moodle/site:config', get_context_instance(CONTEXT_SYSTEM));

$keyword = optional_param('keyword', '', PARAM_RAW);
$autodetect = optional_param('autodetect', '', PARAM_RAW);
$del = optional_param('del', '', PARAM_RAW);
$delall = optional_param('delall', '', PARAM_RAW);
$ignore = optional_param('ignore', '', PARAM_RAW);
$reset = optional_param('reset', '', PARAM_RAW);
$id = optional_param('id', '', PARAM_INT);

$MOODLE2 = ($CFG->version > 2007101600);   /// Used to do some varying Moodle DB stuff later on



// Implement some AJAX calls 

// Delete one user
if (!empty($del) && confirm_sesskey() && ($id != $USER->id)) {
    if (isset($SESSION->users_result[$id])) {
        $user = $SESSION->users_result[$id];
        if (delete_user($user)) {
            unset($SESSION->users_result[$id]);
            echo json_encode(true);
        } else {
            echo json_encode(false);
        }
    } else {
        echo json_encode(false);
    }
    exit;
}

// Delete lots of users
if (!empty($delall) && confirm_sesskey()) {
    if (!empty($SESSION->users_result)) {
        foreach ($SESSION->users_result as $userid => $user) {
            if ($userid != $USER->id) {
                if (delete_user($user)) {
                    unset($SESSION->users_result[$userid]);
                }
            }
        }
    }
    echo json_encode(true);
    exit;
}

if (!empty($ignore)) {
    unset($SESSION->users_result[$id]);
    echo json_encode(true);
    exit;
}

// Print headers and things

print_header('Spam cleaner', 'Spam cleaner');

print_spamcleaner_javascript();

print_box('This script allows you to search all user profiles for certain strings and then delete those accounts which are obviously created by spammers.  You can search for multiple keywords using commas (eg casino,porn)<br /><br />Moodle docs has more information about <a href="http://docs.moodle.org/en/Reducing_spam_in_Moodle">Reducing spam in Moodle</a>.');

print_box_start();     // The forms section at the top

?>

<div style="text-align:center">

<form method="POST">
    <input type="text" name="keyword" id="keyword_el" value="<?php p($keyword) ?>" /> 
    <input type="hidden" name="sesskey" value="<?php echo sesskey();?>" />
    <input type="submit" value="Search for these keywords" />
</form>
<p>eg:  casino, porn, xxx </p>

<hr style="width:60%"/>

<form method="POST">
    <input type="submit" name="autodetect" value="Autodetect common spam patterns" />
</form>


</div>

<?php
print_box_end(); 

echo '<div id="result" style="text-align:center">';

// Print list of resulting profiles

if (!empty($keyword)) {               // Use the keyword(s) supplied by the user
    $keywords = explode(',', $keyword);
    foreach ($keywords as $key => $keyword) {
        $keywords[$key] = trim($keyword);
    }
    search_spammers($keywords);

} else if (!empty($autodetect)) {     // Use the inbuilt keyword list to detect users
    search_spammers($autokeywords);
}

echo '</div>';

print_footer('empty');

/////////////////////////////////////////////////////////////////////////////////


///  Functions 


function search_spammers($keywords) {

    global $CFG, $USER, $DB, $MOODLE2; 

    if (!is_array($keywords)) {
        $keywords = array($keywords);    // Make it into an array
    }

    if ($MOODLE2) {
         $like = $DB->sql_ilike();
    } else {
         $like = sql_ilike();
    }

    $keywordfull = array();
    foreach ($keywords as $keyword) {
        $keyword = addslashes($keyword);   // Just to be safe
        $keywordfull[] = " description $like '%$keyword%' ";
        $keywordfull2[] = " p.summary $like '%$keyword%' ";
    }
    $conditions = '( '.implode(' OR ', $keywordfull).' )';
    $conditions2 = '( '.implode(' OR ', $keywordfull2).' )';

    if ($MOODLE2) {   /// Moodle 2.0 or later has new DB stuff
        $sql = "SELECT * FROM {user} WHERE deleted = 0 AND id <> {$USER->id} AND $conditions";  // Exclude oneself
        $sql2= "SELECT u.*, p.summary FROM {user} AS u, {post} AS p WHERE $conditions2 AND u.id=p.userid AND u.id <> {$USER->id} AND u.deleted = 0";
        $spamusers_desc = $DB->get_recordset_sql($sql);
        $spamusers_blog = $DB->get_recordset_sql($sql2);
    } else {          /// Moodle 1.9.x or earlier
        $sql = "SELECT * FROM {$CFG->prefix}user WHERE deleted = 0 AND id <> {$USER->id} AND $conditions";  // Exclude oneself
        $sql2= "SELECT u.*, p.summary FROM {$CFG->prefix}user AS u, {$CFG->prefix}post AS p WHERE $conditions2 AND u.id=p.userid AND u.id <> {$USER->id} AND u.deleted = 0";
        $spamusers_desc = get_recordset_sql($sql);
        $spamusers_blog = get_recordset_sql($sql2);
    }

    $keywordlist = implode(', ', $keywords);
    print_box('Results of searching user profiles containing: '.s($keywordlist)).' ...';

    print_user_list(array($spamusers_desc, $spamusers_blog), $keywords);

}



function print_user_list($users_rs, $keywords) {
    global $CFG, $SESSION, $MOODLE2;

    // reset session everytime this function is called
    $SESSION->users_result = array();
    $count = 0;

    if ($MOODLE2) {   /// Moodle 2.0 or later has new DB stuff
        foreach ($users_rs as $rs){
            foreach ($rs as $user) {
                if (!$count) {
                    echo '<table border="1" width="100%" id="data-grid"><tr><th>&nbsp;</th><th>User</th><th>Description</th><th>Operation</th></tr>';
                }
                $count++;
                filter_user($user, $keywords, $count);
            }
        }
    } else {
        foreach ($users_rs as $rs) {
            while ($user = rs_fetch_next_record($rs)) {
                if (!$count) {
                    echo '<table border="1" width="100%" id="data-grid"><tr><th>&nbsp;</th><th>User</th><th>Description</th><th>Operation</th></tr>';
                }
                $count++;
                filter_user($user, $keywords, $count);
            }
        }
    }

    if (!$count) {
        echo "No users matching your search";

    } else {
        echo '</table>';
        echo '<div style="text-align:center;margin: 1em">
              <button id="removeall_btn">Delete all these user accounts</button>
              </div>';
    }
}
function filter_user($user, $keywords, $count) {
    global $CFG, $SESSION;

    $smalluserobject = new object;      // All we need to delete them later
    $smalluserobject->id = $user->id;
    $smalluserobject->email = $user->email;
    $smalluserobject->auth = $user->auth;
    $smalluserobject->firstname = $user->firstname;
    $smalluserobject->lastname = $user->lastname;

    if (empty($SESSION->users_result[$user->id])) {
        $SESSION->users_result[$user->id] = $smalluserobject;
    }

    $image_search = false;
    if (in_array('<img', $keywords)) {
        $image_search = true;
    }
    if (isset($user->summary)) {
        $user->description = '<h3>From blog post:</h3>'.$user->summary;
        unset($user->summary);
    }
    if (preg_match('#<img.*src=[\"\']('.$CFG->pixpath.')#', $user->description, $matches)
        && $image_search) {
        $result = false;
        foreach ($keywords as $keyword) {
            if (preg_match('#'.$keyword.'#', $user->description)
                && ($keyword != '<img')) {
                $result = true;
            }
        }
        if ($result) {
            echo print_user_entry($user, $keywords, $count);
        } else {
            unset($user);
        }
    } else {
        echo print_user_entry($user, $keywords, $count);
    }
}


function print_user_entry($user, $keywords, $count) {

    global $SESSION, $CFG;

    $html = '<tr valign="top" id="row-'.$user->id.'" class="result-row">';
    $html .= '<td width="10">'.$count.'</td>';
    $html .= '<td width="30%" align="left"><a href="'.$CFG->wwwroot."/user/view.php?course=1&amp;id=".$user->id.'" title="'.$user->username.'">'.fullname($user).'</a>';

    $html .= "<ul>";
    $profile_set = array('city'=>true, 'country'=>true, 'email'=>true);
    foreach ($profile_set as $key=>$value) {
        if (isset($user->$key)){
            $html .= '<li>'.$user->$key.'</li>';
        }
    }
    $html .= "</ul>";
    $html .= '</td>';

    foreach ($keywords as $keyword) {
        $user->description = format_text(highlight($keyword, $user->description), FORMAT_MOODLE);
    }

    $html .= '<td align="left">'.$user->description.'</td>';
    $html .= '<td width="100px" align="center">';
    $html .= '<button onclick="del_user(this,'.$user->id.')">Delete</button><br />';
    $html .= '<button onclick="ignore_user(this,'.$user->id.')">Ignore</button>';
    $html .= '</td>';
    $html .= '</tr>';
    return $html;
}

function print_spamcleaner_javascript()  {

$sesskey = sesskey();

?>

<script type="text/javascript">
var row = null;
var delall_cb = {
    success: function(o){
        try {
            var resp = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            alert('Unknown but invalid result');
            return;
        }
        if(resp == true){
            window.location.href=window.location.href;
        }
    }
}
function init() {
    YAHOO.util.Event.addListener("removeall_btn", "click", function(){
        var yes = confirm('Are you sure you want to delete all these user accounts?  You can not undo this.');
        if(yes){
            var cObj = YAHOO.util.Connect.asyncRequest('POST', '<?php echo me();?>?delall=yes&sesskey=<?php echo $sesskey;?>', delall_cb); 
        }
    });
}
var del_cb = {
    success: function(o) {
        try {
            var resp = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            alert('Unknown but invalid result');
            return;
        }
        if(row) {
            if(resp == true){
                while(row.tagName != 'TR') {
                    row = row.parentNode;
                }
                row.parentNode.removeChild(row);
                row = null;
            } else {
                alert('Cannot delete this user');
            }
        }
    }
}
var ignore_cb = {
    success: function(o){
        try {
            var resp = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            alert('invalid result');
            return;
        }
        if(row) {
            if(resp == true){
                while(row.tagName != 'TR') {
                    row = row.parentNode;
                }
                row.parentNode.removeChild(row);
                row = null;
            }
        }
    }
}
function del_user(obj, id) {
    var yes = confirm('Are you sure you want to delete this entry?  You can not undo this.');
    if(yes){
        row = obj;
        var cObj = YAHOO.util.Connect.asyncRequest('POST', '<?php echo me();?>?del=yes&sesskey=<?php echo $sesskey;?>&id='+id, del_cb); 
    }
}
function ignore_user(obj, id) {
    row = obj;
    var cObj = YAHOO.util.Connect.asyncRequest('POST', '<?php echo me();?>?ignore=yes&sesskey=<?php echo $sesskey;?>&id='+id, ignore_cb); 
}
YAHOO.util.Event.onDOMReady(init);
</script>

<?php 
}

?>

